@@ -26,5 +26,5 @@
 
 from __future__ import absolute_import, division, print_function
 
-from .builders import LiteratureBuilder  # noqa
+from .builders import LiteratureBuilder, ReferenceBuilder  # noqa
 from .utils import load_schema, validate  # noqa
