@@ -28,3 +28,6 @@ from __future__ import absolute_import, division, print_function
 
 from .literature import LiteratureBuilder  # noqa
 from .references import ReferenceBuilder  # noqa
+from .signatures import SignatureBuilder  # noqa
+from .authors import AuthorBuilder  # noqa
+from .jobs import JobBuilder  # noqa
