@@ -25,38 +25,23 @@
 """Literature builder class and related code."""
 
 from __future__ import absolute_import, division, print_function
+from six import string_types
 
 import warnings
-from functools import wraps
 
 import idutils
 
-from ..utils import (get_license_from_url,
-                     normalize_author_name,
-                     normalize_collaboration,
-                     validate)
+from inspire_utils.date import normalize_date
 
-EMPTIES = [None, '', [], {}]
-
-
-def filter_empty_parameters(func):
-    """Decorator that is filtering empty parameters.
-
-    :param func: function that you want wrapping
-    :type func: function
-    """
-    @wraps(func)
-    def func_wrapper(self, *args, **kwargs):
-        my_kwargs = {key: value for key, value in kwargs.items()
-                     if value not in EMPTIES}
-
-        if (
-                {'source', 'material'}.issuperset(my_kwargs) or not my_kwargs
-        ) and args == ():
-            return
-        return func(self, *args, **my_kwargs)
-
-    return func_wrapper
+from ..utils import (
+    get_license_from_url,
+    normalize_collaboration,
+    normalize_isbn,
+    validate,
+    filter_empty_parameters,
+    EMPTIES
+)
+from .signatures import SignatureBuilder
 
 
 def is_citeable(publication_info):
@@ -66,6 +51,7 @@ def is_citeable(publication_info):
     already populated
     :type publication_info: list
     """
+
     def _item_has_pub_info(item):
         return all(
             key in item for key in (
@@ -90,6 +76,14 @@ def is_citeable(publication_info):
     return has_pub_info and has_page_or_artid
 
 
+def key_already_there(element, elements):
+    """Checks if the given element existng by using the key `key`."""
+    for existing_element in elements:
+        if element['key'] == existing_element['key']:
+            return True
+    return False
+
+
 class LiteratureBuilder(object):
     """Literature record builder."""
 
@@ -199,7 +193,12 @@ class LiteratureBuilder(object):
         """
         if doi is None:
             return
-        doi = idutils.normalize_doi(doi)
+
+        try:
+            doi = idutils.normalize_doi(doi)
+        except AttributeError:
+            return
+
         if not doi:
             return
 
@@ -224,13 +223,14 @@ class LiteratureBuilder(object):
 
     @filter_empty_parameters
     def make_author(self, full_name,
-                    affiliations=None,
-                    roles=None,
-                    raw_affiliations=None,
+                    affiliations=(),
+                    roles=(),
+                    raw_affiliations=(),
                     source=None,
-                    ids=None,
-                    emails=None,
-                    alternative_names=None):
+                    ids=(),
+                    emails=(),
+                    alternative_names=(),
+                    record=None):
         """Make a subrecord representing an author.
 
         Args:
@@ -246,75 +246,34 @@ class LiteratureBuilder(object):
                 elements are of the form ``(schema, value)``.
             emails(List[str]): email addresses of the author.
             alternative_names(List[str]): alternative names of the author.
-
+            record(dict): reference to the author record
         Returns:
             dict: a schema-compliant subrecord.
         """
-        def _add_affiliations(author, affiliations):
-            if affiliations is None:
-                return
+        builder = SignatureBuilder()
+        builder.set_full_name(full_name)
+        builder.set_record(record)
 
-            author.setdefault('affiliations', [])
-            for affiliation in filter(bool, affiliations):
-                author['affiliations'].append({
-                    'value': affiliation
-                })
+        for affiliation in affiliations:
+            builder.add_affiliation(affiliation)
 
-        def _add_raw_affiliations(author, raw_affiliations, source):
-            if raw_affiliations is None:
-                return
+        for role in roles:
+            builder.add_inspire_role(role)
 
-            author.setdefault('raw_affiliations', [])
-            for raw_affiliation in filter(bool, raw_affiliations):
-                author['raw_affiliations'].append(self._sourced_dict(
-                    source,
-                    value=raw_affiliation,
-                ))
+        for raw_affiliation in raw_affiliations:
+            builder.add_raw_affiliation(raw_affiliation, source or self.source)
 
-        def _add_emails(author, emails):
-            if emails is None:
-                return
+        for id_schema, id_value in ids:
+            if id_schema and id_value:
+                builder.set_uid(id_value, schema=id_schema)
 
-            author.setdefault('emails', []).extend(filter(bool, emails))
+        for email in emails:
+            builder.add_email(email)
 
-        def _add_ids(author, ids):
-            if ids is None:
-                return
+        for alternative_name in alternative_names:
+            builder.add_alternative_name(alternative_name)
 
-            author.setdefault('ids', [])
-            for schema, value in ids:
-                if value:
-                    author['ids'].append({
-                        'schema': schema,
-                        'value': value,
-                    })
-
-        def _add_alternative_names(author, alternative_names):
-            if alternative_names is None:
-                return
-
-            author.setdefault('alternative_names', []).extend(
-                filter(bool, alternative_names)
-            )
-
-        def _add_inspire_roles(author, inspire_roles):
-            if inspire_roles is None:
-                return
-
-            author.setdefault('inspire_roles', []).extend(filter(bool, roles))
-
-        author = {}
-
-        author['full_name'] = normalize_author_name(full_name)
-
-        _add_affiliations(author, affiliations)
-        _add_raw_affiliations(author, raw_affiliations, source)
-        _add_emails(author, emails)
-        _add_ids(author, ids)
-        _add_alternative_names(author, alternative_names)
-        _add_inspire_roles(author, roles)
-
-        return author
+        return builder.obj
 
     @filter_empty_parameters
     def add_book(self, publisher=None, place=None, date=None):
@@ -328,15 +287,16 @@ class LiteratureBuilder(object):
         :param place: place of publication
         :type place: string
 
-        :param date: the date of publication
-        :type date: date
+        :param date: A (partial) date in any format.
+            The date should contain at least a year
+        :type date: string
 
         :rtype: dict
         """
 
         imprint = {}
         if date is not None:
-            imprint['date'] = date
+            imprint['date'] = normalize_date(date)
         if place is not None:
             imprint['place'] = place
         if publisher is not None:
@@ -352,7 +312,7 @@ class LiteratureBuilder(object):
         """
         isbn_dict = {}
         if isbn is not None:
-            isbn_dict['value'] = isbn
+            isbn_dict['value'] = normalize_isbn(isbn)
         if medium is not None:
             isbn_dict['medium'] = medium
 
@@ -445,6 +405,9 @@ class LiteratureBuilder(object):
         pubinfo_freetext=None,
         material=None,
         parent_record=None,
+        parent_isbn=None,
+        journal_record=None,
+        conference_record=None,
     ):
         """Add publication info.
 
@@ -483,17 +446,40 @@ class LiteratureBuilder(object):
         :type material: string
 
         :param parent_record: reference for the parent record
-        :type parent_record: string
+        :type parent_record: dict/string
+
+        :param parent_isbn: isbn for the parent record
+        :type parent_isbn: string
+
+        :param journal_record: reference for the journal record
+        :type journal_record: dict
+
+        :param conference_record: reference for the conference record
+        :type conference_record: dict
         """
+
+        # If only journal title is present, and no other fields, assume the
+        # paper was submitted, but not yet published
+        if journal_title and all(
+            not field for field in (cnum, artid, journal_issue,
+                                    journal_volume, page_start, page_end)):
+            self.add_public_note('Submitted to {}'.format(journal_title))
+            return
+
         publication_item = {}
         for key in ('cnum', 'artid', 'page_end', 'page_start',
                     'journal_issue', 'journal_title',
-                    'journal_volume', 'year', 'pubinfo_freetext', 'material'):
+                    'journal_volume', 'year', 'pubinfo_freetext', 'material',
+                    'journal_record', 'conference_record'):
             if locals()[key] is not None:
                 publication_item[key] = locals()[key]
         if parent_record is not None:
-            parent_item = {'$ref': parent_record}
-            publication_item['parent_record'] = parent_item
+            # TODO: remove `if string` check [BREAKING] while bumping major version
+            if isinstance(parent_record, string_types):
+                parent_record = {'$ref': parent_record}
+            publication_item['parent_record'] = parent_record
+        if parent_isbn is not None:
+            publication_item['parent_isbn'] = normalize_isbn(parent_isbn)
         if page_start and page_end:
             try:
                 self.add_number_of_pages(
@@ -511,19 +497,21 @@ class LiteratureBuilder(object):
     def add_imprint_date(self, imprint_date):
         """Add imprint date.
 
-        :type imprint_date: string. A formatted date is required (yyyy-mm-dd)
+        :type imprint_date: string. A (partial) date in any format.
+            The date should contain at least a year
         """
         self._append_to('imprints', {
-            'date': imprint_date
+            'date': normalize_date(imprint_date)
         })
 
     @filter_empty_parameters
     def add_preprint_date(self, preprint_date):
         """Add preprint date.
 
-        :type preprint_date: string. A formatted date is required (yyyy-mm-dd)
+        :type preprint_date: string. A (partial) date in any format.
+            The date should contain at least a year
         """
-        self.record['preprint_date'] = preprint_date
+        self.record['preprint_date'] = normalize_date(preprint_date)
 
     @filter_empty_parameters
     def add_thesis(
@@ -572,6 +560,22 @@ class LiteratureBuilder(object):
             'legacy_name': legacy_name
         })
 
+    @filter_empty_parameters
+    def add_accelerator_experiment(self, legacy_name, record=None):
+        """Add legacy name in accelerator experiment.
+
+        :type legacy_name: string
+
+        :param record: reference to the experiment record
+        :type record: dict
+        """
+        experiment = {'legacy_name': legacy_name}
+
+        if record is not None:
+            experiment['record'] = record
+
+        self._append_to('accelerator_experiments', experiment)
+
     @filter_empty_parameters
     def add_language(self, language):
         """Add language.
@@ -656,7 +660,7 @@ class LiteratureBuilder(object):
         self._append_to('titles', title_entry)
 
     @filter_empty_parameters
-    def add_title_translation(self, title, language=None, source=None):
+    def add_title_translation(self, title, language, source=None):
         """Add title translation.
 
         :param title: translated title
@@ -671,9 +675,8 @@ class LiteratureBuilder(object):
         title_translation = self._sourced_dict(
             source,
             title=title,
+            language=language,
         )
-        if language is not None:
-            title_translation['language'] = language
 
         self._append_to('title_translations', title_translation)
 
@@ -874,58 +877,77 @@ class LiteratureBuilder(object):
         """Set curated flag."""
         self.record['curated'] = curated
 
-    @filter_empty_parameters
-    def add_figure(
-        self,
-        key,
-        url,
-        caption=None,
-        label=None,
-        material=None,
-        source=None,
-    ):
-        """Add a figure."""
-        figure = self._sourced_dict(source)
-
-        if key:
-            figure['key'] = key
+    def _check_metadata_for_file(self, **kwargs):
+        file = self._sourced_dict(kwargs.get('source'))
+        if kwargs.get('key'):
+            file['key'] = kwargs['key']
         else:
             raise TypeError("Required argument 'key' should not be 'falsey'.")
-        if url:
-            figure['url'] = url
+        if kwargs.get('url'):
+            file['url'] = kwargs['url']
         else:
             raise TypeError("Required argument 'url' should not be 'falsey'.")
-
-        for dict_key in ('caption', 'label', 'material'):
-            if locals()[dict_key] is not None:
-                figure[dict_key] = locals()[dict_key]
-
-        if figure:
-            self._append_to('figures', figure)
+        return file
 
     @filter_empty_parameters
-    def add_document(
-        self,
-        key,
-        url,
-        description=None,
-        fulltext=None,
-        hidden=None,
-        material=None,
-        original_url=None,
-        source=None,
-    ):
-        """Add a document."""
-        document = self._sourced_dict(source)
+    def add_figure(self, key, url, **kwargs):
+        """Add a figure.
 
-        if key:
-            document['key'] = key
-        else:
-            raise TypeError("Required argument 'key' should not be 'falsey'.")
-        if url:
-            document['url'] = url
-        else:
-            raise TypeError("Required argument 'url' should not be 'falsey'.")
+        Args:
+            key (string): document key
+            url (string): document url
+        Keyword Args:
+            caption (string): simple description
+            label (string):
+            material (string):
+            original_url (string): original url
+            filename (string): current url
+
+        Returns: None
+
+        """
+        figure = self._check_metadata_for_file(key=key, url=url, **kwargs)
+
+        for dict_key in (
+            'caption',
+            'label',
+            'material',
+            'filename',
+            'url',
+            'original_url',
+        ):
+            if kwargs.get(dict_key) is not None:
+                figure[dict_key] = kwargs[dict_key]
+
+        if key_already_there(figure, self.record.get('figures', ())):
+            raise ValueError(
+                'There\'s already a figure with the key %s.'
+                % figure['key']
+            )
+
+        self._append_to('figures', figure)
+        self.add_document
+
+    @filter_empty_parameters
+    def add_document(self, key, url, **kwargs):
+        """
+        Adds document to record
+        Args:
+            key (string): document key
+            url (string): document url
+        Keyword Args:
+            description (string): simple description
+            fulltext (bool): mark if this is a full text
+            hidden (bool): is document should be hidden
+            material (string):
+            original_url (string): original url
+            filename (string): current url
+
+
+        Returns: None
+
+        """
+        document = self._check_metadata_for_file(key=key, url=url, **kwargs)
 
         for dict_key in (
             'description',
@@ -934,9 +956,24 @@ class LiteratureBuilder(object):
             'material',
             'original_url',
             'url',
+            'filename',
         ):
-            if locals()[dict_key] is not None:
-                document[dict_key] = locals()[dict_key]
+            if kwargs.get(dict_key):
+                document[dict_key] = kwargs[dict_key]
 
-        if document:
-            self._append_to('documents', document)
+        if key_already_there(document, self.record.get('documents', ())):
+            raise ValueError(
+                'There\'s already a document with the key %s.'
+                % document['key']
+            )
+
+        self._append_to('documents', document)
+
+    @filter_empty_parameters
+    def add_reference(self, reference):
+        """Add reference to references field.
+
+        :param reference: reference dictionary, see :class:`.ReferenceBuilder`
+        :type reference: dict
+        """
+        self._append_to('references', reference)
