@@ -28,21 +28,27 @@ import copy
 import json
 import os
 import re
+from functools import partial, wraps
 
+import idutils
+import rfc3987
 import six
+from bleach.linkifier import LinkifyFilter
+from bleach.sanitizer import Cleaner
+from idutils import is_orcid
 from inspire_utils.date import PartialDate
+from isbn import ISBN
+from jsonschema import Draft4Validator, RefResolver, draft4_format_checker
 from jsonschema import validate as jsonschema_validate
-from jsonschema import RefResolver, draft4_format_checker
-from nameparser import HumanName
-from nameparser.config import Constants
 from pkg_resources import resource_filename
+from six.moves.urllib.parse import urlsplit
 from unidecode import unidecode
 
-from six.moves.urllib.parse import urlsplit
-
-from .errors import SchemaKeyNotFound, SchemaNotFound
+from .errors import (SchemaKeyNotFound, SchemaNotFound, SchemaUIDConflict,
+                     UnknownUIDSchema)
 
 _schema_root_path = os.path.abspath(resource_filename(__name__, 'records'))
+_resolved_schema_root_path = os.path.abspath(resource_filename(__name__, 'resolved_records'))
 
 _RE_2_CHARS = re.compile(r'[a-z].*[a-z]', re.IGNORECASE)
 _RE_CHAR = re.compile(r'[a-z]', re.IGNORECASE)
@@ -53,11 +59,14 @@ _RE_COLLABORATION_LEADING = re.compile(
 _RE_COLLABORATION_TRAILING = re.compile(
     r'\bcollaborations?\s*$', re.IGNORECASE
 )
+_RE_PUBLIC_DOMAIN_URL = re.compile(
+    r'^/publicdomain/zero(?:/(?P<version>[\.\d]*))?'
+)
 _RE_LICENSE_URL = re.compile(
     r'^/licenses/(?P<sublicense>[-\w]*)(?:/(?P<version>[\.\d]*))?'
 )
 _RE_VOLUME_STARTS_WITH_A_LETTER = re.compile(
-    r'^(?P<letter>[A-Z])(?P<volume>\d+)', re.IGNORECASE
+    r'^(?P<letter>[A-Z])(?P<volume>\d[\dA-Z]*$)', re.IGNORECASE
 )
 _RE_VOLUME_ENDS_WITH_A_LETTER = re.compile(
     r'(?P<volume>\d+)(?P<letter>[A-Z])$', re.IGNORECASE
@@ -66,6 +75,31 @@ _RE_TITLE_ENDS_WITH_A_LETTER = re.compile(
     r'(?P<title>.+(\.| ))(?P<letter>[A-Z])$', re.IGNORECASE
 )
 
+_RE_AUTHORS_UID = {
+    'CERN': (re.compile(r'^(CCID-|CERN-)?(?P<uid>\d+)$', flags=re.I), 'CERN-{}'),
+    'JACOW': (re.compile(r'^(JACOW-)?(?P<uid>\d{8})$', flags=re.I), 'JACoW-{}'),
+    'SLAC': (re.compile(r'^(SLAC-)?(?P<uid>\d+)$', flags=re.I), 'SLAC-{}'),
+    'DESY': (re.compile(r'^(DESY-)?(?P<uid>\d+)$', flags=re.I), 'DESY-{}'),
+    'INSPIRE ID': (re.compile(r'^(INSPIRE-)?(?P<uid>\d{8})$', flags=re.I), 'INSPIRE-{}'),
+    'INSPIRE BAI': (re.compile(r'^(?P<uid>((\w|\-|\')+\.)+\d+)$'), '{}'),
+}
+
+# Matches new style arXiv ID, with an old-style class specification
+# (Malformed, but appears in APS records)
+RE_ARXIV_POST_2007_CLASS = re.compile(
+    "(arxiv:)?((?P<category>(?:[a-z\-]+)(?:\.[a-z]{2})?)/)?(?P<identifier>\d{4}\.\d{4,5})(v\d+)?$",
+    flags=re.I
+)
+
+RE_ARXIV_PRE_2007_CLASS = re.compile(
+    "(arxiv:)?(?P<category>(?P<extraidentifier>[a-z\-]+)(?:\.[a-z]{2})?)/(?P<identifier>\d{4}\d+)(v\d+)?$",
+    flags=re.I
+)
+
+JOURNALS_IGNORED_IN_OLD_TO_NEW = [
+    'econf',
+]
+
 
 # list produced from https://arxiv.org/archive/
 _NEW_CATEGORIES = {
@@ -81,7 +115,7 @@ _NEW_CATEGORIES = {
     'comp-gas': 'nlin.CG',
     'dg-ga': 'math.DG',
     'funct-an': 'math.FA',
-    'mtrl-th': 'cont-mat.mtrl-sci',
+    'mtrl-th': 'cond-mat.mtrl-sci',
     'patt-sol': 'nlin.PS',
     'plasm-ph': 'physics.plasm-ph',
     'q-alg': 'math.QA',
@@ -220,7 +254,7 @@ ARXIV_TO_INSPIRE_CATEGORY_MAPPING = {
     'quant-ph': 'General Physics',
 }
 
-_JOURNALS_ALREADY_ENDING_WITH_A_LETTER = set([
+_JOURNALS_ALREADY_ENDING_WITH_A_LETTER = {
     'Acta Cryst.A',
     'Acta Cryst.B',
     'Acta Cryst.D',
@@ -266,12 +300,116 @@ _JOURNALS_ALREADY_ENDING_WITH_A_LETTER = set([
     'Spectrochim.Acta A',
     'Tellus A',
     'Trans.Int.Astron.Union A',
-])
+}
 
 _JOURNALS_THAT_NEED_A_HIDDEN_PUBNOTE = {
     'Phys.Lett.B': set(str(el) for el in range(24, 171)),
 }
 
+_JOURNALS_RENAMED_OLD_TO_NEW = {
+    'Ann.Inst.H.Poincare Anal.Non Lineaire': 'Ann.Inst.H.Poincare C Anal.Non Lineaire',
+    'Annales Soc.Sci.Brux.Ser.I Sci.Math.Astron.Phys.': 'Annales Soc.Sci.Bruxelles.I',
+    'Annales Soc.Sci.Bruxelles Ser.B Sci.Phys.Nat.': 'Annales Soc.Sci.Bruxelles B',
+    'Diss.Abstr.Int.': 'Diss.Abstr.Int.B',
+    'J.Comb.Theory Ser.': 'J.Comb.Theor.A',
+    'J.Vac.Sci.Technol.A Vac.Surf.Films': 'J.Vac.Sci.Technol.A',
+    'J.Vac.Sci.Technol.B Microelectron.Nanometer Struct.': 'J.Vac.Sci.Technol.B',
+    'Nucl.Phys.Proc.Suppl.': 'Nucl.Phys.B Proc.Suppl.',
+    'Proc.Roy.Irish Acad.(Sect.A)': 'Proc.Roy.Irish Acad.A',
+    'Proc.Roy.Soc.Lond.': 'Proc.Roy.Soc.Lond.A',
+    'Univ.Politech.Bucharest Sci.Bull.': 'Univ.Politech.Bucharest Sci.Bull.A',
+}
+_JOURNALS_RENAMED_NEW_TO_OLD = {v: k for (k, v) in six.iteritems(_JOURNALS_RENAMED_OLD_TO_NEW)}
+
+_JOURNALS_WITH_YEAR_ADDED_TO_VOLUME = {
+    'JHEP',
+    'JCAP',
+}
+
+EMPTIES = [None, '', [], {}]
+
+_BLEACH_CONFIG = {
+    'tags': ['a', 'b', 'br', 'em', 'i', 'li', 'ol', 'p', 'strong', 'ul'],
+    'attributes': {'a': ['href', 'title']},
+    'strip': True,
+    'filters': [LinkifyFilter],
+}
+_bleach_cleaner = Cleaner(**_BLEACH_CONFIG)
+
+
+def filter_empty_parameters(func):
+    """Decorator that is filtering empty parameters.
+
+    :param func: function that you want wrapping
+    :type func: function
+    """
+    @wraps(func)
+    def func_wrapper(self, *args, **kwargs):
+        my_kwargs = {key: value for key, value in kwargs.items()
+                     if value not in EMPTIES}
+        args_is_empty = all(arg in EMPTIES for arg in args)
+
+        if (
+                {'source', 'material'}.issuperset(my_kwargs) or not my_kwargs
+        ) and args_is_empty:
+            return
+        return func(self, *args, **my_kwargs)
+
+    return func_wrapper
+
+
+def author_id_normalize_and_schema(uid, schema=None):
+    """Detect and normalize an author UID schema.
+
+    Args:
+        uid (string): a UID string
+        schema (string): try to resolve to schema
+
+    Returns:
+        Tuple[string, string]: a tuple (uid, schema) where:
+        - uid: the UID normalized to comply with the id.json schema
+        - schema: a schema of the UID or *None* if not recognised
+
+    Raise:
+        UnknownUIDSchema: if UID is too little to definitively guess the schema
+        SchemaUIDConflict: if specified schema is not matching the given UID
+    """
+    def _get_uid_normalized_in_schema(_uid, _schema):
+        regex, template = _RE_AUTHORS_UID[_schema]
+        match = regex.match(_uid)
+        if match:
+            return template.format(match.group('uid'))
+
+    if idutils.is_orcid(uid) and schema in (None, 'ORCID'):
+        return idutils.normalize_orcid(uid), 'ORCID'
+
+    if schema and schema not in _RE_AUTHORS_UID:
+        # Schema explicitly specified, but this function can't handle it
+        raise UnknownUIDSchema(uid)
+
+    if schema:
+        normalized_uid = _get_uid_normalized_in_schema(uid, schema)
+        if normalized_uid:
+            return normalized_uid, schema
+        else:
+            raise SchemaUIDConflict(schema, uid)
+
+    match_schema, normalized_uid = None, None
+    for candidate_schema in _RE_AUTHORS_UID:
+        candidate_uid = _get_uid_normalized_in_schema(uid, candidate_schema)
+        if candidate_uid:
+            if match_schema:
+                # Valid against more than one candidate schema, ambiguous
+                raise UnknownUIDSchema(uid)
+            match_schema = candidate_schema
+            normalized_uid = candidate_uid
+
+    if match_schema:
+        return normalized_uid, match_schema
+
+    # No guessess have been found
+    raise UnknownUIDSchema(uid)
+
 
 def normalize_arxiv_category(category):
     """Normalize arXiv category to be schema compliant.
@@ -282,7 +420,7 @@ def normalize_arxiv_category(category):
 
     Example:
         >>> from inspire_schemas.utils import normalize_arxiv_category
-        >>> normalize_arxiv_category('funct-an')
+        >>> normalize_arxiv_category('funct-an')  # doctest: +SKIP
         u'math.FA'
 
     """
@@ -383,19 +521,22 @@ def split_pubnote(pubnote_str):
     return {key: val for (key, val) in six.iteritems(pubnote) if val is not None}
 
 
-def build_pubnote(title, volume, page_start, page_end, artid):
+def build_pubnote(title, volume, page_start=None, page_end=None, artid=None):
     """Build pubnote string from parts (reverse of split_pubnote)."""
-    pubnote = None
-    if title and volume:
-        pubnote = '{},{}'.format(title, volume)
-        if page_start and page_end:
-            pubnote += ',{}-{}'.format(page_start, page_end)
-        elif page_start:
-            pubnote += ',{}'.format(page_start)
-        if artid and artid != page_start:
-            pubnote += ',{}'.format(artid)
+    if title and volume and artid and artid != page_start:
+        pubnote_format = u'{title},{volume},{artid}'
+    elif title and volume and page_start and page_end:
+        pubnote_format = u'{title},{volume},{page_start}-{page_end}'
+    elif title and volume and page_start:
+        pubnote_format = u'{title},{volume},{page_start}'
+    elif title and volume:
+        pubnote_format = u'{title},{volume}'
+    else:
+        return None
 
-    return pubnote
+    return pubnote_format.format(
+        title=title, volume=volume, page_start=page_start, page_end=page_end, artid=artid
+    )
 
 
 class LocalRefResolver(RefResolver):
@@ -411,15 +552,22 @@ class LocalRefResolver(RefResolver):
             )
 
 
-def get_schema_path(schema):
+def get_schema_path(schema, resolved=False):
     """Retrieve the installed path for the given schema.
 
-    :param schema: String with the (relative or absolute) url of the
-        schema to validate, for example, 'records/authors.json' or 'jobs.json',
-        or by just the name like 'jobs'.
-    :type schema: str
-    :return: The path or the given schema name.
-    :rtype: str
+    Args:
+        schema(str): relative or absolute url of the schema to validate, for
+            example, 'records/authors.json' or 'jobs.json', or just the name of the
+            schema, like 'jobs'.
+        resolved(bool): if True, the returned path points to a fully resolved
+            schema, that is to the schema with all `$ref` replaced by their
+            targets.
+
+    Returns:
+        str: path to the given schema name.
+
+    Raises:
+        SchemaNotFound: if no schema could be found.
     """
     def _strip_first_path_elem(path):
         """Pass doctests.
@@ -448,7 +596,10 @@ def get_schema_path(schema):
 
     path = _schema_to_normalized_path(schema)
     while path:
-        schema_path = os.path.abspath(os.path.join(_schema_root_path, path))
+        if resolved:
+            schema_path = os.path.abspath(os.path.join(_resolved_schema_root_path, path))
+        else:
+            schema_path = os.path.abspath(os.path.join(_schema_root_path, path))
         if os.path.exists(schema_path):
             return os.path.abspath(schema_path)
 
@@ -457,23 +608,56 @@ def get_schema_path(schema):
     raise SchemaNotFound(schema=schema)
 
 
-def load_schema(schema_name):
+def load_schema(schema_name, resolved=False):
     """Load the given schema from wherever it's installed.
 
-    :param schema_name: Name of the schema to load, for example 'authors'.
+    Args:
+        schema_name(str): Name of the schema to load, for example 'authors'.
+        resolved(bool): If True will return the resolved schema, that is with
+            all the $refs replaced by their targets.
+
+    Returns:
+        dict: the schema with the given name.
     """
     schema_data = ''
-    with open(get_schema_path(schema_name)) as schema_fd:
+    with open(get_schema_path(schema_name, resolved)) as schema_fd:
         schema_data = json.loads(schema_fd.read())
 
-    if '$schema' not in schema_data:
-        schema_data = {'$schema': schema_data}
-
     return schema_data
 
 
 inspire_format_checker = draft4_format_checker
 inspire_format_checker.checks('date', raises=ValueError)(PartialDate.loads)
+inspire_format_checker.checks('uri-reference', raises=ValueError)(
+    partial(rfc3987.parse, rule='URI_reference')
+)
+inspire_format_checker.checks('orcid')(is_orcid)
+
+
+def _load_schema_for_record(data, schema=None):
+    """Load the schema from a given record.
+
+    Args:
+        data (dict): record data.
+        schema (Union[dict, str]): schema to validate against.
+
+    Returns:
+        dict: the loaded schema.
+
+    Raises:
+        SchemaNotFound: if the given schema was not found.
+        SchemaKeyNotFound: if ``schema`` is ``None`` and no ``$schema`` key was
+            found in ``data``.
+        jsonschema.SchemaError: if the schema is invalid.
+    """
+    if schema is None:
+        if '$schema' not in data:
+            raise SchemaKeyNotFound(data=data)
+        schema = data['$schema']
+
+    if isinstance(schema, six.string_types):
+        schema = load_schema(schema_name=schema)
+    return schema
 
 
 def validate(data, schema=None):
@@ -494,13 +678,7 @@ def validate(data, schema=None):
         jsonschema.SchemaError: if the schema is invalid.
         jsonschema.ValidationError: if the data is invalid.
     """
-    if schema is None:
-        if '$schema' not in data:
-            raise SchemaKeyNotFound(data=data)
-        schema = data['$schema']
-
-    if isinstance(schema, six.string_types):
-        schema = load_schema(schema_name=schema)
+    schema = _load_schema_for_record(data, schema)
 
     return jsonschema_validate(
         instance=data,
@@ -510,70 +688,33 @@ def validate(data, schema=None):
     )
 
 
-def normalize_author_name(author):
-    """Normalize author name.
+def get_validation_errors(data, schema=None):
+    """Validation errors for a given record.
 
-    :param author: author name
-    :type author: string
+    Args:
+        data (dict): record to validate.
+        schema (Union[dict, str]): schema to validate against. If it is a
+            string, it is intepreted as the name of the schema to load (e.g.
+            ``authors`` or ``jobs``). If it is ``None``, the schema is taken
+            from ``data['$schema']``. If it is a dictionary, it is used
+            directly.
+    Yields:
+        jsonschema.exceptions.ValidationError: validation errors.
 
-    :return name: the name of the author normilized
+    Raises:
+        SchemaNotFound: if the given schema was not found.
+        SchemaKeyNotFound: if ``schema`` is ``None`` and no ``$schema`` key was
+            found in ``data``.
+        jsonschema.SchemaError: if the schema is invalid.
     """
-    constants = Constants()
-    roman_numeral_suffixes = [u'v', u'vi', u'vii', u'viii', u'ix', u'x',
-                              u'xii', u'xiii', u'xiv', u'xv']
-    titles = [u'Dr', u'Prof', u'Professor', u'Sir', u'Editor', u'Ed', u'Mr',
-              u'Mrs', u'Ms', u'Chair', u'Co-Chair', u'Chairs', u'co-Chairs']
-    constants.titles.remove(*constants.titles).add(*titles)
-    constants.suffix_not_acronyms.add(*roman_numeral_suffixes)
+    schema = _load_schema_for_record(data, schema)
 
-    def _is_initial(author_name):
-        return len(author_name) == 1 or u'.' in author_name
-
-    def _ensure_dotted_initials(author_name):
-        if _is_initial(author_name)\
-                and u'.' not in author_name:
-            seq = (author_name, u'.')
-            author_name = u''.join(seq)
-        return author_name
-
-    def _ensure_dotted_suffixes(author_suffix):
-        if u'.' not in author_suffix:
-            seq = (author_suffix, u'.')
-            author_suffix = u''.join(seq)
-        return author_suffix
-
-    def _is_roman_numeral(suffix):
-        """Controls that the userinput only contains valid roman numerals"""
-        valid_roman_numerals = [u'M', u'D', u'C', u'L', u'X',
-                                u'V', u'I', u'(', u')']
-        return all(letters in valid_roman_numerals
-                   for letters in suffix.upper())
-
-    name = HumanName(author, constants=constants)
-
-    name.first = _ensure_dotted_initials(name.first)
-    name.middle = _ensure_dotted_initials(name.middle)
-
-    if _is_initial(name.first) and _is_initial(name.middle):
-        normalized_names = u'{first_name}{middle_name}'
-    else:
-        normalized_names = u'{first_name} {middle_name}'
-
-    normalized_names = normalized_names.format(
-        first_name=name.first,
-        middle_name=name.middle,
+    errors = Draft4Validator(
+        schema,
+        resolver=LocalRefResolver.from_schema(schema),
+        format_checker=inspire_format_checker
     )
-
-    if _is_roman_numeral(name.suffix):
-        suffix = name.suffix.upper()
-    else:
-        suffix = _ensure_dotted_suffixes(name.suffix)
-
-    final_name = u', '.join(
-        part for part in (name.last, normalized_names.strip(), suffix)
-        if part)
-
-    return final_name
+    return errors.iter_errors(data)
 
 
 def normalize_collaboration(collaboration):
@@ -593,6 +734,10 @@ def normalize_collaboration(collaboration):
     if not collaboration:
         return []
 
+    collaboration = collaboration.strip()
+    if collaboration.startswith('(') and collaboration.endswith(')'):
+        collaboration = collaboration[1:-1]
+
     collaborations = _RE_AND.split(collaboration)
     collaborations = (_RE_COLLABORATION_LEADING.sub('', collab)
                       for collab in collaborations)
@@ -620,9 +765,17 @@ def get_license_from_url(url):
     split_url = urlsplit(url, scheme='http')
 
     if split_url.netloc.lower() == 'creativecommons.org':
-        license = ['CC']
-        match = _RE_LICENSE_URL.match(split_url.path)
-        license.extend(part.upper() for part in match.groups() if part)
+        if 'publicdomain' in split_url.path:
+            match = _RE_PUBLIC_DOMAIN_URL.match(split_url.path)
+            if match is None:
+                license = ['public domain']
+            else:
+                license = ['CC0']
+                license.extend(part for part in match.groups() if part)
+        else:
+            license = ['CC']
+            match = _RE_LICENSE_URL.match(split_url.path)
+            license.extend(part.upper() for part in match.groups() if part)
     elif split_url.netloc == 'arxiv.org':
         license = ['arXiv']
         match = _RE_LICENSE_URL.match(split_url.path)
@@ -676,20 +829,43 @@ def convert_old_publication_info_to_new(publication_infos):
 
     for publication_info in publication_infos:
         _publication_info = copy.deepcopy(publication_info)
-
         journal_title = _publication_info.get('journal_title')
+
+        try:
+            journal_title = _JOURNALS_RENAMED_OLD_TO_NEW[journal_title]
+            _publication_info['journal_title'] = journal_title
+            result.append(_publication_info)
+            continue
+        except KeyError:
+            pass
+
         journal_volume = _publication_info.get('journal_volume')
-        if journal_title and journal_volume:
+
+        if journal_title in _JOURNALS_WITH_YEAR_ADDED_TO_VOLUME and journal_volume and len(journal_volume) == 4:
+            try:
+                was_last_century = int(journal_volume[:2]) > 50
+            except ValueError:
+                pass
+            else:
+                _publication_info['year'] = int('19' + journal_volume[:2] if was_last_century else '20' + journal_volume[:2])
+                _publication_info['journal_volume'] = journal_volume[2:]
+            result.append(_publication_info)
+            continue
+
+        if journal_title and journal_volume and journal_title.lower() not in JOURNALS_IGNORED_IN_OLD_TO_NEW:
             volume_starts_with_a_letter = _RE_VOLUME_STARTS_WITH_A_LETTER.match(journal_volume)
             volume_ends_with_a_letter = _RE_VOLUME_ENDS_WITH_A_LETTER.match(journal_volume)
             match = volume_starts_with_a_letter or volume_ends_with_a_letter
             if match:
                 _publication_info.pop('journal_record', None)
-                _publication_info['journal_title'] = ''.join([
-                    journal_title,
-                    '' if journal_title.endswith('.') else ' ',
-                    match.group('letter'),
-                ])
+                if journal_title in _JOURNALS_RENAMED_OLD_TO_NEW.values():
+                    _publication_info['journal_title'] = journal_title
+                else:
+                    _publication_info['journal_title'] = ''.join([
+                        journal_title,
+                        '' if journal_title.endswith('.') else ' ',
+                        match.group('letter'),
+                    ])
                 _publication_info['journal_volume'] = match.group('volume')
 
         hidden = _publication_info.pop('hidden', None)
@@ -729,9 +905,26 @@ def convert_new_publication_info_to_old(publication_infos):
 
     for publication_info in publication_infos:
         _publication_info = copy.deepcopy(publication_info)
-
         journal_title = _publication_info.get('journal_title')
+
+        try:
+            journal_title = _JOURNALS_RENAMED_NEW_TO_OLD[journal_title]
+            _publication_info['journal_title'] = journal_title
+            result.append(_publication_info)
+            continue
+        except KeyError:
+            pass
+
         journal_volume = _publication_info.get('journal_volume')
+        year = _publication_info.get('year')
+
+        if (journal_title in _JOURNALS_WITH_YEAR_ADDED_TO_VOLUME and year and
+                journal_volume and len(journal_volume) == 2):
+            two_digit_year = str(year)[2:]
+            _publication_info['journal_volume'] = ''.join([two_digit_year, journal_volume])
+            result.append(_publication_info)
+            continue
+
         if journal_title and journal_volume:
             match = _RE_TITLE_ENDS_WITH_A_LETTER.match(journal_title)
             if match and _needs_a_hidden_pubnote(journal_title, journal_volume):
@@ -749,3 +942,100 @@ def convert_new_publication_info_to_old(publication_infos):
         result.append(_publication_info)
 
     return result
+
+
+def fix_url_bars_instead_of_slashes(string):
+    """A common error in urls is that all ``/`` have been changed for ``|``, we fix that in this function"""
+    if string[:7] == 'http:||' or string[:8] == 'https:||':
+        string = string.replace('|', '/')
+    return string
+
+
+def fix_url_add_http_if_missing(string):
+    """Add the starting ``http`` to a url that is missing it"""
+    if string[:3] == 'www':
+        string = 'http://' + string
+    return string
+
+
+def fix_url_replace_tilde(string):
+    """Replace unicode characters by their working equivalent"""
+    string = string.replace('\u223c', '~')
+    return string.replace('\u02dc', '~')
+
+
+def fix_reference_url(url):
+    """Used to parse an incorect url to try to fix it with the most common ocurrences for errors.
+    If the fixed url is still incorrect, it returns ``None``.
+
+    Returns:
+        String containing the fixed url or the original one if it could not be fixed.
+    """
+    new_url = url
+
+    new_url = fix_url_bars_instead_of_slashes(new_url)
+
+    new_url = fix_url_add_http_if_missing(new_url)
+
+    new_url = fix_url_replace_tilde(new_url)
+
+    try:
+        rfc3987.parse(new_url, rule="URI")
+        return new_url
+    except ValueError:
+        return url
+
+
+def normalize_isbn(isbn):
+    """Normalize an ISBN in order to be schema-compliant."""
+    try:
+        return str(ISBN(isbn))
+    except Exception:
+        return isbn
+
+
+def is_arxiv(obj):
+    """Return ``True`` if ``obj`` contains an arXiv identifier.
+
+    The ``idutils`` library's ``is_arxiv`` function has been
+    modified here to work with two regular expressions instead
+    of three and adding a check for valid arxiv categories only"""
+    arxiv_test = obj.split()
+    if not arxiv_test:
+        return False
+
+    matched_arxiv = (RE_ARXIV_PRE_2007_CLASS.match(arxiv_test[0]) or
+                     RE_ARXIV_POST_2007_CLASS.match(arxiv_test[0]))
+
+    if not matched_arxiv:
+        return False
+
+    if not matched_arxiv.group('category'):
+        return True
+
+    valid_arxiv_categories_lower = [category.lower() for category in valid_arxiv_categories()]
+    category = matched_arxiv.group('category').lower()
+    return (category in valid_arxiv_categories_lower or
+            category.replace('-', '.') in valid_arxiv_categories_lower)
+
+
+def normalize_arxiv(obj):
+    """Return a normalized arXiv identifier from ``obj``."""
+    obj = obj.split()[0]
+
+    matched_arxiv_pre = RE_ARXIV_PRE_2007_CLASS.match(obj)
+    if matched_arxiv_pre:
+        return ('/'.join(matched_arxiv_pre.group("extraidentifier", "identifier"))).lower()
+
+    matched_arxiv_post = RE_ARXIV_POST_2007_CLASS.match(obj)
+    if matched_arxiv_post:
+        return matched_arxiv_post.group("identifier")
+
+    return None
+
+
+def sanitize_html(text):
+    """Sanitize HTML for use inside records fields.
+
+    This strips most of the tags and attributes, only allowing a safe whitelisted subset."""
+    return _bleach_cleaner.clean(text)
